import pytest
from unittest.mock import AsyncMock, patch
from vllm_judge import Judge, EvaluationResult, Metric
from vllm_judge.exceptions import InvalidInputError, MetricNotFoundError, ParseError


class TestJudgeInitialization:
    """Test Judge initialization and basic functionality."""
    
    def test_judge_init_with_config(self, mock_config):
        """Test Judge initialization with config."""
        judge = Judge(mock_config)
        assert judge.config == mock_config
        assert judge.client is not None
        assert judge.metrics == {}
    
    def test_judge_from_url(self, monkeypatch):
        """Test Judge.from_url class method."""
        monkeypatch.setattr(
            "vllm_judge.client.detect_model_sync", 
            lambda url: "detected-model"
        )
        
        judge = Judge.from_url("http://localhost:8000")
        assert judge.config.base_url == "http://localhost:8000"
        assert judge.config.model == "detected-model"
    
    async def test_judge_context_manager(self, mock_judge):
        """Test Judge as async context manager."""
        async with mock_judge as judge:
            assert judge is not None
        # Should have called close (mocked)


class TestJudgeEvaluation:
    """Test Judge evaluation methods."""
    
    async def test_basic_evaluation(self, mock_judge):
        """Test basic evaluation with criteria."""
        result = await mock_judge.evaluate(
            content="The Earth is round.",
            criteria="scientific accuracy"
        )
        
        assert isinstance(result, EvaluationResult)
        assert result.decision == "GOOD"
        assert result.reasoning == "Test reasoning"
        assert result.score == 8.0
    
    async def test_evaluation_with_metric(self, mock_judge):
        """Test evaluation using a predefined metric."""
        # Create a test metric
        metric = Metric(
            name="test_metric",
            criteria="test criteria",
            rubric="test rubric",
            scale=(1, 10)
        )
        
        result = await mock_judge.evaluate(
            content="Test content",
            metric=metric
        )
        
        assert isinstance(result, EvaluationResult)
    
    async def test_evaluation_with_templates(self, mock_judge):
        """Test evaluation with template variables."""
        result = await mock_judge.evaluate(
            content="Python is a programming language",
            criteria="Evaluate this {doc_type} for {audience}",
            template_vars={
                "doc_type": "statement",
                "audience": "beginners"
            }
        )
        
        assert isinstance(result, EvaluationResult)
        # Check that template variables were added to metadata
        assert "template_vars" in result.metadata
    
    async def test_comparison_evaluation(self, mock_judge):
        """Test comparison evaluation."""
        result = await mock_judge.evaluate(
            content={"a": "Response A", "b": "Response B"},
            criteria="Which response is better?"
        )
        
        assert isinstance(result, EvaluationResult)
    
    async def test_evaluation_missing_criteria(self, mock_judge):
        """Test evaluation without criteria should raise error."""
        with pytest.raises(InvalidInputError):
            await mock_judge.evaluate(content="Test content")
    
    async def test_evaluation_with_input(self, mock_judge):
        """Test evaluation with input parameter."""
        result = await mock_judge.evaluate(
            content="Paris",
            input="What is the capital of France?",
            criteria="accuracy"
        )
        
        assert isinstance(result, EvaluationResult)


class TestJudgeConvenienceMethods:
    """Test Judge convenience methods."""
    
    async def test_score_method(self, mock_judge):
        """Test the score convenience method."""
        result = await mock_judge.score(
            criteria="clarity",
            content="The explanation is clear.",
            scale=(1, 5)
        )
        
        assert isinstance(result, EvaluationResult)
    
    async def test_qa_evaluate_method(self, mock_judge):
        """Test the qa_evaluate convenience method."""
        result = await mock_judge.qa_evaluate(
            question="What is 2+2?",
            answer="4",
            criteria="mathematical accuracy"
        )
        
        assert isinstance(result, EvaluationResult)
    
    async def test_compare_method(self, mock_judge):
        """Test the compare convenience method."""
        result = await mock_judge.compare(
            response_a="Response A is better",
            response_b="Response B is worse", 
            criteria="quality"
        )
        
        assert isinstance(result, EvaluationResult)
    
    async def test_classify_method(self, mock_judge):
        """Test the classify convenience method."""
        result = await mock_judge.classify(
            content="This is spam email",
            categories=["spam", "not_spam"],
            criteria="email classification"
        )
        
        assert isinstance(result, EvaluationResult)


class TestJudgeMetricManagement:
    """Test Judge metric management."""
    
    def test_register_metric(self, mock_judge):
        """Test registering a custom metric."""
        metric = Metric(name="custom_metric", criteria="test criteria")
        mock_judge.register_metric(metric)
        
        assert "custom_metric" in mock_judge.metrics
        assert mock_judge.metrics["custom_metric"] == metric
    
    def test_get_metric_custom(self, mock_judge):
        """Test getting a custom registered metric."""
        metric = Metric(name="custom_metric", criteria="test criteria")
        mock_judge.register_metric(metric)
        
        retrieved = mock_judge.get_metric("custom_metric")
        assert retrieved == metric
    
    def test_get_metric_builtin(self, mock_judge):
        """Test getting a built-in metric."""
        # This should work with built-in metrics
        try:
            metric = mock_judge.get_metric("HELPFULNESS")
            assert isinstance(metric, Metric)
        except MetricNotFoundError:
            # If built-in metrics aren't loaded, that's also fine for this test
            pass
    
    def test_get_metric_not_found(self, mock_judge):
        """Test getting a non-existent metric."""
        with pytest.raises(MetricNotFoundError):
            mock_judge.get_metric("nonexistent_metric")
    
    def test_list_metrics(self, mock_judge):
        """Test listing available metrics."""
        # Register a custom metric
        metric = Metric(name="custom_metric", criteria="test")
        mock_judge.register_metric(metric)
        
        metrics = mock_judge.list_metrics()
        assert "custom_metric" in metrics
        # Should also include built-in metrics
        assert len(metrics) > 1


class TestJudgeBatchProcessing:
    """Test Judge batch processing."""
    
    async def test_batch_evaluate(self, mock_judge):
        """Test batch evaluation."""
        data = [
            {"content": "Text 1", "criteria": "quality"},
            {"content": "Text 2", "criteria": "accuracy"},
            {"content": "Text 3", "criteria": "clarity"}
        ]
        
        with patch.object(mock_judge, 'evaluate') as mock_evaluate:
            mock_evaluate.return_value = EvaluationResult(
                decision="GOOD", reasoning="Test"
            )
            
            # Mock BatchProcessor
            with patch('vllm_judge.judge.BatchProcessor') as mock_batch_processor:
                from vllm_judge.models import BatchResult
                mock_processor_instance = AsyncMock()
                mock_processor_instance.process.return_value = BatchResult(
                    results=[
                        EvaluationResult(decision="GOOD", reasoning="Test"),
                        EvaluationResult(decision="OK", reasoning="Test"),
                        EvaluationResult(decision="EXCELLENT", reasoning="Test")
                    ],
                    total=3,
                    successful=3,
                    failed=0,
                    duration_seconds=1.0
                )
                mock_batch_processor.return_value = mock_processor_instance
                
                result = await mock_judge.batch_evaluate(data)
                
                assert isinstance(result, BatchResult)
                assert result.total == 3
                assert result.successful == 3


class TestJudgeResponseParsing:
    """Test Judge response parsing."""
    
    def test_parse_response_valid_json(self, mock_judge):
        """Test parsing valid JSON response."""
        response = '{"decision": "GOOD", "reasoning": "Clear and accurate"}'
        result = mock_judge._parse_response(response)
        
        assert result.decision == "GOOD"
        assert result.reasoning == "Clear and accurate"
    
    def test_parse_response_json_in_markdown(self, mock_judge):
        """Test parsing JSON wrapped in markdown code blocks."""
        response = '''
        Here's the evaluation:
        ```json
        {"decision": "EXCELLENT", "reasoning": "Very good response"}
        ```
        '''
        result = mock_judge._parse_response(response)
        
        assert result.decision == "EXCELLENT"
        assert result.reasoning == "Very good response"
    
    def test_parse_response_missing_decision(self, mock_judge):
        """Test parsing response missing decision field."""
        response = '{"reasoning": "Good response"}'
        
        with pytest.raises(ParseError):
            mock_judge._parse_response(response)
    
    def test_parse_response_missing_reasoning(self, mock_judge):
        """Test parsing response missing reasoning field."""
        response = '{"decision": "GOOD"}'
        result = mock_judge._parse_response(response)
        
        assert result.decision == "GOOD"
        assert result.reasoning == "No reasoning provided"
    
    def test_parse_response_invalid_json(self, mock_judge):
        """Test parsing invalid JSON response."""
        response = "This is not JSON at all"
        
        with pytest.raises(ParseError):
            mock_judge._parse_response(response)